from ..base import BaseExporter
from prometheus_client import Gauge
import os
import re
import shutil
import subprocess
import json

class CustomExporter(BaseExporter):
    def generate(self):
        disk_avail_bytes = Gauge('container_disk_avail_bytes', '', ['filesystem', 'machine'], registry=self.registry)
        disk_avail_inodes = Gauge('container_disk_avail_inodes', '', ['filesystem', 'machine'], registry=self.registry)
        disk_used_bytes = Gauge('container_disk_used_bytes', '', ['filesystem', 'machine'], registry=self.registry)
        disk_used_inodes = Gauge('container_disk_used_inodes', '', ['filesystem', 'machine'], registry=self.registry)
        proc_mount_broken = Gauge('container_proc_mount_broken', '', ['machine'], registry=self.registry)
        pod_data = self._get_pods()
        pod_names = [d['Names'][0] for d in pod_data if 'Names' in d and len(d['Names'])]
        xfs_data = self._get_xfs_quota()
        for pod in pod_names:
            target = os.readlink("/podman/container_data/%s/root" % pod)
            disk_usage_info = shutil.disk_usage(target)
            disk_used_bytes.labels('rootfs', pod).set(disk_usage_info.used)
            disk_avail_bytes.labels('rootfs', pod).set(disk_usage_info.free)

            # TODO CHECK for accuracy
            disk_stats = os.statvfs(target)
            disk_avail_inodes.labels('rootfs', pod).set(disk_stats.f_ffree)
            disk_used_inodes.labels('rootfs', pod).set(disk_stats.f_files - disk_stats.f_ffree)

            if pod in xfs_data:
                pod_xfs_data = xfs_data[pod]
                disk_used_bytes.labels('home', pod).set(pod_xfs_data['bytes_used'])
                disk_avail_bytes.labels('home', pod).set((pod_xfs_data['bytes_total'] - pod_xfs_data['bytes_used']))
                disk_avail_inodes.labels('home', pod).set((pod_xfs_data['inodes_total'] - pod_xfs_data['inodes_used']))
                disk_used_inodes.labels('home', pod).set(pod_xfs_data['inodes_used'])

        container_pid_map = {d['Names'][0]: d['Pid'] for d in pod_data if 'Names' in d and len(d['Names'])}
        meminfo_data = self._nsenter_command(
            container_pid_map=container_pid_map,
            command=["wc", "-l", "/proc/meminfo"]
        )
        for pod, result in meminfo_data.items():
            if re.search('^\s*[0-9]',result):
                proc_mount_broken.labels(pod).set(0)
            else:
                proc_mount_broken.labels(pod).set(1)

    def _get_pods(self):
        try:
            result = subprocess.run(["podman", "ps", "--format", "json"], capture_output=True, text=True)
            data = json.loads(result.stdout)
        except Exception:
            raise Exception("failed to get pods")
        return data


    def _get_xfs_quota(self):
        script = "xfs_quota -x -c 'report -p -bi' /podman"
        result = subprocess.run(["bash", "-c", script], capture_output=True, text=True)
        data = {}

        for line in result.stdout.strip().splitlines():
            xfs_fields = line.split()
            machine = xfs_fields[0]
            if machine.startswith('vps') or machine.startswith('sql'):
                data[machine] = {
                    'bytes_used': float(xfs_fields[1]),
                    'bytes_total': float(xfs_fields[3]),
                    'inodes_used': float(xfs_fields[6]),
                    'inodes_total': float(xfs_fields[8])
                    }
        return data

    # container_pid_map
    # {container_name: pid}
    def _nsenter_command(self, container_pid_map={}, command=[]):
		# TODO multiprocess.pool
        result_dict = {}
        for pod, pid in  container_pid_map.items():
            result = subprocess.run(["nsenter", "-m", "-C", "-t", str(pid), "--", *command], capture_output=True, text=True)
            result_dict[pod] = result.stdout
        return result_dict
